package com.example.catchTheLetters.service.impl;

import com.example.catchTheLetters.config.MinioConfig;
import com.example.catchTheLetters.constant.CommonConstant;
import com.example.catchTheLetters.entity.Version;
import com.example.catchTheLetters.entity.VersionDownload;
import com.example.catchTheLetters.service.VersionService;
import com.example.catchTheLetters.utils.R;
import io.minio.*;
import io.minio.errors.*;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.List;

@Service
public class VersionServiceImpl implements VersionService {

    @Resource
    private MongoTemplate mongoTemplate;

    @Resource
    private MinioClient minioClient;
    @Autowired
    private MinioConfig minioConfig;

    @Override
    public Long getVersion(String tableName) {
        var res = mongoTemplate.findOne(new Query(Criteria.where("table").is(tableName)), Version.class);
        return res == null ? null : res.getVersion();
    }

    @Override
    public void setVersion(String tableName, Long version) {
        var res = mongoTemplate.findOne(new Query(Criteria.where("table").is(tableName)), Version.class);
        if (res == null) {
            mongoTemplate.save(new Version(tableName, version));
        } else {
            res.setVersion(version);
            mongoTemplate.save(res);
        }
    }

    @Override
    public R<String> addVersionDownload(Long time, String version, String url, MultipartFile file) {
        VersionDownload versionDownload = new VersionDownload();
        versionDownload.setUrl(url);
        versionDownload.setTime(time);
        versionDownload.setVersion(version);
        VersionDownload insert = mongoTemplate.insert(versionDownload);
        if (insert.getId() == null){
            return R.fail("添加数据库失败");
        }
        //TODO

        return R.ok(insert.getId());
    }

    @Override
    public R<List<VersionDownload>> getNewDownloads() {
        Query query = new Query()
                .with(Sort.by(Sort.Direction.DESC,"time"))
                .limit(5);
        List<VersionDownload> list = mongoTemplate.find(query, VersionDownload.class);
        return R.ok(list);
    }

    @Override
    public String testMinIO(MultipartFile file) throws Exception {
        System.out.println(minioConfig.getBucket());
        minioClient.makeBucket(MakeBucketArgs.builder().bucket(minioConfig.getBucket()).build());
//        boolean bucketFound = minioClient.bucketExists(BucketExistsArgs.builder().bucket(minioConfig.getBucket()).build());
        return "未找到";
    }
}
